/*
 *
 * Copyright (C) 2007 SpinetiX S.A.
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------------
 *
 */

/* LED control on National Semiconductor DP3848J Ethernet PHY */

#include <common.h>
#include <status_led.h>
#include <miiphy.h>

#ifdef CONFIG_STATUS_LED

#if ! ( defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII) )
#error LED support requires MII support to be enabled (CONFIG_MII or CFG_CMD_MII)
#endif

#ifndef CFG_FAULT_MII_ADDR
#error CFG_FAULT_MII_ADDR needs to be defined to the PHY MII address
#endif

#define LEDCR_REG_ADDR		0x18
#define LEDCR_DRV_SHIFT		3

/* NOTE: LEDs are inverted, must write a 1 to register to turn off and
 * 0 to turn on !! */

static unsigned short ledcr_reg;

static void led_set(led_id_t mask, int state, int in_init)
{
	const unsigned short old_ledcr = ledcr_reg;
	switch (state) {
	case STATUS_LED_OFF:
		ledcr_reg |= mask;
		break;
	case STATUS_LED_BLINKING:
		printf("Warning: LED blinking not supported, using 'ON' state\n");
		/* fall-through */
	case STATUS_LED_ON:
		ledcr_reg &= ~mask;
	}
	if (old_ledcr == ledcr_reg && ! in_init)
		return; /* nothing to do */
	if (miiphy_write(miiphy_get_current_dev(), CFG_FAULT_MII_ADDR,
			 LEDCR_REG_ADDR, ledcr_reg))
		printf("Warning: failed updating LED control register on Ethernet PHY\n");
}

void __led_init(led_id_t mask, int state)
{
	ledcr_reg |= ( mask << LEDCR_DRV_SHIFT );
	led_set(mask, state, 1);
}

void __led_toggle(led_id_t mask)
{
	ledcr_reg ^= mask;
	if (miiphy_write(miiphy_get_current_dev(), CFG_FAULT_MII_ADDR,
			 LEDCR_REG_ADDR, ledcr_reg))
		printf("Warning: failed updating LED control register on Ethernet PHY\n");
}

void __led_set(led_id_t mask, int state)
{
	led_set(mask, state, 0);
}

#ifdef CONFIG_SHOW_BOOT_PROGRESS

void show_boot_progress(int progress)
{
	if (progress >= 0) {
		status_led_set(STATUS_LED_RED, STATUS_LED_OFF);
		status_led_set(STATUS_LED_GREEN, STATUS_LED_OFF);
		udelay(25 * 1000);
		status_led_set(STATUS_LED_GREEN, STATUS_LED_ON);
	} else {
		status_led_set(STATUS_LED_GREEN, STATUS_LED_OFF);
		status_led_set(STATUS_LED_RED, STATUS_LED_ON);
	}
}

#endif /* CONFIG_SHOW_BOOT_PROGRESS */

#endif /* CONFIG_STATUS_LED */
